class CheckRun:
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self):
        self._results = []

    ##################################################################################################
    # Methods
    ##################################################################################################
    def run(self, dataset, checks):
        self._results = []
        for check in checks:
            result = check.run(dataset)
            self._results.append(result)

    ##################################################################################################
    # Properties
    ##################################################################################################
    @property
    def results(self):
        return self._results

    @property
    def start(self):
        starts = (result.start for result in self.results)
        start = min(starts)
        return start

    @property
    def end(self):
        ends = (result.end for result in self.results)
        end = min(ends)
        return end

    @property
    def duration(self):
        duration = self.end - self.start
        return duration

    @property
    def pass_rate(self):
        checks = len(self.results)
        if checks == 0:
            return 0
        check_results = list(result.passed for result in self.results)
        passed = check_results.count(True)
        rate = passed / checks
        return rate

    @property
    def fail_rate(self):
        checks = len(self.results)
        if checks == 0:
            return 0
        check_results = list(result.passed for result in self.results)
        failed = check_results.count(False)
        rate = failed / checks
        return rate



from .base.BaseCheck import BaseCheck
from .base.BaseColumnCheck import BaseColumnCheck
from .base.BaseFunctionCheck import BaseFunctionCheck


class CheckColumnMatchStrftime(BaseCheck, BaseColumnCheck, BaseFunctionCheck):
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, id, column_name, datetime_format):
        column_function_name = "check_column_match_strftime_format"

        BaseCheck.__init__(self, id)
        BaseColumnCheck.__init__(self, column_name)
        BaseFunctionCheck.__init__(self, column_function_name)
        
        self.datetime_format = datetime_format

    ##################################################################################################
    # Methods
    ##################################################################################################
    def _run(self, dataset):
        passed = self.column_function(dataset, self.column_name, self.datetime_format).all()
        message = f"All values in column '{self.column_name}' comply with the given function '{self.column_function_str}' for datetime format '{self.datetime_format}'!"
        if not passed:
            message = f"Not all values in column '{self.column_name}' comply with the given function '{self.column_function_str}' for datetime format '{self.datetime_format}'!"
        return passed, message

    ##################################################################################################
    # Properties
    ##################################################################################################
    @property
    def datetime_format(self):
        return self._datetime_format

    @datetime_format.setter
    def datetime_format(self, value):
        self._datetime_format = value
        self._settings["datetime_format"] = value



from .base.BaseCheck import BaseCheck
from .base.BaseColumnCheck import BaseColumnCheck
from .base.BaseValueCheck import BaseValueCheck


class CheckColumnAllSame(BaseCheck, BaseColumnCheck, BaseValueCheck):
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, id, column_name, column_value=None):
        BaseCheck.__init__(self, id)
        BaseColumnCheck.__init__(self, column_name)
        BaseValueCheck.__init__(self, column_value)

    ##################################################################################################
    # Methods
    ##################################################################################################
    def _run(self, dataset):
        if self.column_value is None:
            passed = dataset[self.column_name].nunique() == 1
            message = f"All values in column '{self.column_name}' are the same!"
            if not passed:
                message = f"Not all values in column '{self.column_name}' are the same!"
            return passed, message
        else:
            passed = (dataset[self.column_name] == self.column_value).all()
            message = f"All values in column '{self.column_name}' have the value '{self.column_value}'!"
            if not passed:
                message = f"Not all values in column '{self.column_name}' have the value '{self.column_value}'!"
            return passed, message



from sys import path
path.append('../unswamp/')

from helpers import TestHelpers
from unswamp.checks import (
    CheckColumnAllSame,
)
from unittest import TestCase, main



class TestCheckColumnAllSame(TestCase):
    def test_CheckColumnAllSame_properties(self):
        check_id = TestHelpers.str_random()
        col = TestHelpers.str_random()
        val = TestHelpers.str_random()
        check = CheckColumnAllSame(check_id, col, val)

        TestHelpers.test_property(self, check, "id", check_id)
        TestHelpers.test_property(self, check, "column_name", col)
        TestHelpers.test_property(self, check, "column_value", val)

    def test_CheckColumnAllSame_run(self):
        check_id = TestHelpers.str_random()
        col = "Col_Same"
        val = TestHelpers.same
        check = CheckColumnAllSame(check_id, col, val)
        dataset = TestHelpers.get_dataset()
        result = check.run(dataset)
        self.assertTrue(
            result.passed, f"Non expected check result for check '{type(check)}' with message '{result.message}'")


if __name__ == '__main__':
    main()



from sys import path
path.append('../unswamp/')

from helpers import TestHelpers
from unswamp.checks import (
    CheckColumnFunctionAll,
    Functions,
)
from unittest import TestCase, main


class TestCheckColumnFunctionAll(TestCase):
    def test_CheckColumnFunctionAll_properties(self):
        check_id = TestHelpers.str_random()
        col = TestHelpers.str_random()
        val = TestHelpers.str_random()
        # TODO: use mock
        func = TestHelpers.function_name
        check = CheckColumnFunctionAll(check_id, col, val, func)

        TestHelpers.test_property(self, check, "id", check_id)
        TestHelpers.test_property(self, check, "column_name", col)
        TestHelpers.test_property(self, check, "column_value", val)
        TestHelpers.test_property(self, check, "column_function_name", func)

    def test_CheckColumnFunctionAll_run(self):
        check_id = TestHelpers.str_random()
        col = "Col_Same"
        val = TestHelpers.same
        func = "check_column_equal"
        check = CheckColumnFunctionAll(check_id, col, val, func)
        dataset = TestHelpers.get_dataset()
        result = check.run(dataset)
        self.assertTrue(
            result.passed, f"Non expected check result for check '{type(check)}' with message '{result.message}'")


if __name__ == '__main__':
    main()



import inspect
import re
from datetime import datetime
from dateutil.parser import parse


class Functions:
    lambda_functions = {
        "check_column_equal": lambda dataset, col, val: dataset[col] == val,
        "check_column_not_equal": lambda dataset, col, val: dataset[col] != val,
        "check_column_lower_than": lambda dataset, col, val: dataset[col] < val,
        "check_column_greater_than": lambda dataset, col, val: dataset[col] > val,
        "check_column_lower_equal_than": lambda dataset, col, val: dataset[col] <= val,
        "check_column_greater_equal_than": lambda dataset, col, val: dataset[col] >= val,
        "check_column_between": lambda dataset, col, vals: dataset[col].apply(lambda x: x > vals[0] and x < vals[1]),
        "check_column_in_set": lambda dataset, col, vals: dataset[col].apply(lambda x: x in vals),
        "check_column_not_in_set": lambda dataset, col, vals: dataset[col].apply(lambda x: x not in vals),
        "check_column_length_between": lambda dataset, col, vals: dataset[col].apply(lambda x: len(str(x)) >= vals[0] and len(str(x)) <= vals[1]),
        "check_column_length_equal": lambda dataset, col, val: dataset[col].apply(lambda x: len(str(x)) == val),
        "check_column_match_regex": lambda dataset, col, rgx: dataset[col].apply(lambda x: True if re.fullmatch(rgx, str(x)) else False),
        "check_column_not_match_regex": lambda dataset, col, rgx: dataset[col].apply(lambda x: False if re.fullmatch(rgx, str(x)) else True),
        "check_column_match_strftime_format": lambda dataset, col, format: dataset[col].apply(lambda x: Functions.str_2_datetime(str(x), format)),
        "check_column_dateutil_parseable": lambda dataset, col: dataset[col].apply(lambda x: Functions.str_datetime_parseable(str(x))),
    }

    @staticmethod
    def get_string(function):
        desc = inspect.getsource(function)
        desc = desc.strip()
        return desc

    @staticmethod
    def like_2_regex(like):
        # TODO: Match pattern in between %text%text%
        # TODO: Replace all regex chars
        # https://codereview.stackexchange.com/questions/36861/convert-sql-like-to-regex/36864
        # bcd → ^bcd$
        # %bcd → ^.*?bcd$
        # bcd% → ^bcd.*?$
        # %bcd% → ^.*?bcd.*?$
        regex_chars = ["^", ".", "*", "?", "$"]
        regex = ""
        regex_start = "^"
        regex_end = "$"
        if like.startswith("%"):
            regex_start = "^.*?"
            like = like[1:]
        if like.endswith("%"):
            regex_end = ".*?$"
            like = like[:-1]

        for char in regex_chars:
            like = like.replace(char, f"[{char}]")
        regex = f"{regex_start}{like}{regex_end}"
        return regex

    @staticmethod
    def str_2_datetime(date_text, strftime):
        # https://stackoverflow.com/questions/16870663/how-do-i-validate-a-date-string-format-in-python/16870699
        try:
            datetime.strptime(date_text, strftime)
            return True
        except ValueError:
            return False

    @staticmethod
    def str_datetime_parseable(date_text):
        # https://stackoverflow.com/questions/16870663/how-do-i-validate-a-date-string-format-in-python/16870699
        try:
            parse(date_text)
            return True
        except ValueError:
            return False



from .CheckColumnAllSame import CheckColumnAllSame
from .CheckColumnFunctionAll import CheckColumnFunctionAll
from .CheckColumnIsNotNull import CheckColumnIsNotNull
from .CheckColumnIsNull import CheckColumnIsNull
from .CheckColumnIsUnique import CheckColumnIsUnique
from .CheckColumnRegexAll import CheckColumnRegexAll
from .CheckColumnLikeAll import CheckColumnLikeAll
from .CheckColumnMatchStrftime import CheckColumnMatchStrftime
from .CheckColumnDateutilParseable import CheckColumnDateutilParseable
from .Functions import Functions


import os


class CheckResult:
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, id, check, settings, start, end, passed, message):
        self.id = id
        self.check = check
        self.settings = settings
        self.start = start
        self.end = end
        self.passed = passed
        self.message = message

    ##################################################################################################
    # Overrides
    ##################################################################################################
    def __str__(self):
        obj = "#" * 50 + os.linesep
        obj += f"# {self.id}{os.linesep}"
        obj += "#" * 50 + os.linesep
        obj += f"- passed: '{self.passed}'{os.linesep}"
        obj += f"- message: '{self.message}'{os.linesep}"
        obj += "#" * 50 + os.linesep
        obj += f"- check: '{self.check}'{os.linesep}"
        obj += f"- start: '{self.start}'{os.linesep}"
        obj += f"- end: '{self.end}'{os.linesep}"
        obj += f"- duration: '{self.duration}'{os.linesep}"
        obj += "#" * 50 + os.linesep
        obj += f"- settings:{os.linesep}"
        for key in self.settings:
            obj += f"    - {key}: '{self.settings[key]}'{os.linesep}"
        obj += "#" * 50 + os.linesep
        obj += os.linesep
        return obj

    ##################################################################################################
    # Properties
    ##################################################################################################
    @property
    def id(self):
        return self._id

    @id.setter
    def id(self, value):
        self._id = value

    @property
    def check(self):
        return self._check

    @check.setter
    def check(self, value):
        self._check = value

    @property
    def settings(self):
        return self._settings

    @settings.setter
    def settings(self, value):
        self._settings = value

    @property
    def start(self):
        return self._start

    @start.setter
    def start(self, value):
        self._start = value

    @property
    def end(self):
        return self._end

    @end.setter
    def end(self, value):
        self._end = value

    @property
    def passed(self):
        return self._passed

    @passed.setter
    def passed(self, value):
        self._passed = value

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def duration(self):
        return self.end - self.start



from ..Functions import Functions


class BaseFunctionCheck():
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, column_function_name):
        self.column_function_name = column_function_name

    ##################################################################################################
    # Properties
    ##################################################################################################
    @property
    def column_function_name(self):
        return self._column_function_name

    @column_function_name.setter
    def column_function_name(self, value):
        self._column_function_name = value
        self._settings["column_function_name"] = value
        self._settings["column_function_str"] = self.column_function_str

    @property
    def column_function(self):
        func = Functions.lambda_functions[self.column_function_name]
        return func

    @property
    def column_function_str(self):
        func = self.column_function
        desc = Functions.get_string(func)
        return desc



# https://github.com/pypa/sampleproject/blob/main/setup.py
from setuptools import setup, find_packages


with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()


setup(
    name="unswamp",
    version="1.0.4",
    author="Stefan Kaspar",
    author_email="me@fullbox.ch",
    description="A python package for data quality unit testing.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://gitlab.com/debugair/unswamp",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Quality Assurance",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    keywords="data, quality, test, unittest",
    python_requires=">=3.6",
    package_data={

    },
    project_urls={
        'Bug Reports': 'https://gitlab.com/debugair/unswamp/-/issues',
        'Source': 'https://gitlab.com/debugair/unswamp',
    },
)



from sys import path
path.append('../unswamp/')

from unittest import TestCase, main
from unswamp.checks import (
    CheckColumnLikeAll,
)
from helpers import TestHelpers


class TestCheckColumnLikeAll(TestCase):
    def test_CheckColumnLikeAll_properties(self):
        check_id = TestHelpers.str_random()
        col = TestHelpers.str_random()
        like = TestHelpers.str_random()
        check = CheckColumnLikeAll(check_id, col, like)

        TestHelpers.test_property(self, check, "id", check_id)
        TestHelpers.test_property(self, check, "column_name", col)
        TestHelpers.test_property(self, check, "column_like", like)

    def test_CheckColumnLikeAll_run(self):
        check_id = TestHelpers.str_random()
        col = "Col_Same"
        like = f"%{TestHelpers().same}%"
        check = CheckColumnLikeAll(check_id, col, like)
        dataset = TestHelpers.get_dataset()
        result = check.run(dataset)
        self.assertTrue(result.passed, f"Non expected check result for check '{type(check)}' with message '{result.message}'")


if __name__ == '__main__':
    main()



from .base.BaseCheck import BaseCheck
from .base.BaseColumnCheck import BaseColumnCheck
from .base.BaseFunctionCheck import BaseFunctionCheck


class CheckColumnRegexAll(BaseCheck, BaseColumnCheck, BaseFunctionCheck):
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, id, column_name, column_regex, match=True):
        column_function_name = "check_column_match_regex"
        if not match:
            column_function_name = "check_column_not_match_regex"

        BaseCheck.__init__(self, id)
        BaseColumnCheck.__init__(self, column_name)
        BaseFunctionCheck.__init__(self, column_function_name)
        
        self.column_regex = column_regex
        self.match = match

    ##################################################################################################
    # Methods
    ##################################################################################################

    def _run(self, dataset):
        passed = self.column_function(
            dataset, self.column_name, self.column_regex).all()
        match_message = "match"
        if self.match == False:
            match_message = "don't match"
        message = f"All values in column '{self.column_name}' {match_message} the given regex pattern '{self.column_regex}'!"
        if not passed:
            message = f"Not all values in column '{self.column_name}' {match_message} the given regex pattern '{self.column_regex}'!"
        return passed, message

    ##################################################################################################
    # Properties
    ##################################################################################################
    @property
    def column_regex(self):
        return self._column_regex

    @column_regex.setter
    def column_regex(self, value):
        self._column_regex = value
        self._settings["column_regex"] = value

    @property
    def match(self):
        return self._match

    @match.setter
    def match(self, value):
        self._match = value
        self._settings["match"] = value



import string
from os import linesep
from io import StringIO
from datetime import datetime
import pandas as pd

from random import seed, random, randint, choice, SystemRandom


class TestHelpers:
    curr_seed=42
    same="A"
    records=10
    datetime_format="%Y-%m-%d"
    regex_year = r"^[0-9]{4}$"
    function_name = "check_column_equal"

    @staticmethod
    def build_csv(records=1000000, curr_seed=42, same="A"):
        seed(curr_seed)
        csv = f"Col_Id,Col_Empty,Col_Same,Col_Year,Col_Gender,Col_Export{linesep}"
        for pos in range(records):
            year = randint(1901, datetime.today().year)
            gender = "M" if random() > 0.5 else "F"
            export = datetime.today().strftime(TestHelpers.datetime_format)
            csv += f"{pos},,{same},{year},{gender},{export}{linesep}"
        return csv

    @staticmethod
    def build_dataset(csv):
        data = StringIO(csv)
        dataset = pd.read_csv(data)
        return dataset

    @staticmethod
    def get_dataset():
        csv = TestHelpers.build_csv(TestHelpers.records, TestHelpers.curr_seed, TestHelpers.same)
        dataset = TestHelpers.build_dataset(csv)
        return dataset

    @staticmethod
    def str_random(length=10):
        rand = "".join(SystemRandom().choice(
            string.ascii_uppercase + string.digits) for _ in range(length))
        return rand

    @staticmethod
    def test_property(test, obj, property, expectation):
        value = getattr(obj, property)
        test.assertEqual(value, expectation, f"Property '{property}' of object '{type(obj)}' should be '{expectation}' but was '{value}'!")



from .base.BaseCheck import BaseCheck
from .base.BaseColumnCheck import BaseColumnCheck
from .base.BaseValueCheck import BaseValueCheck
from .base.BaseFunctionCheck import BaseFunctionCheck


class CheckColumnFunctionAll(BaseCheck, BaseColumnCheck, BaseValueCheck, BaseFunctionCheck):
    ##################################################################################################
    # Constructor
    ##################################################################################################
    def __init__(self, id, column_name, column_value, column_function_name):
        BaseCheck.__init__(self, id)
        BaseColumnCheck.__init__(self, column_name)
        BaseValueCheck.__init__(self, column_value)
        BaseFunctionCheck.__init__(self, column_function_name)

    ##################################################################################################
    # Methods
    ##################################################################################################

    def _run(self, dataset):
        passed = self.column_function(
            dataset, self.column_name, self.column_value).all()
        message = f"All values in column '{self.column_name}' comply with the given function '{self.column_function_str}' for value '{self.column_value}'!"
        if not passed:
            message = f"Not all values in column '{self.column_name}' comply with the given function '{self.column_function_str}' for value '{self.column_value}'!"
        return passed, message



from sys import path
path.append('../unswamp/')

from unittest import TestCase, main
from unswamp.checks import (
    CheckColumnMatchStrftime,
)
from helpers import TestHelpers


class TestCheckColumnMatchStrftime(TestCase):
    def test_CheckColumnMatchStrftime_properties(self):
        check_id = TestHelpers.str_random()
        col = TestHelpers.str_random()
        datetime_format = TestHelpers.str_random()
        check = CheckColumnMatchStrftime(check_id, col, datetime_format)

        TestHelpers.test_property(self, check, "id", check_id)
        TestHelpers.test_property(self, check, "column_name", col)
        TestHelpers.test_property(self, check, "datetime_format", datetime_format)

    def test_CheckColumnMatchStrftime_run(self):
        check_id = TestHelpers.str_random()
        col = "Col_Export"
        datetime_format = TestHelpers.datetime_format
        check = CheckColumnMatchStrftime(check_id, col, datetime_format)
        dataset = TestHelpers.get_dataset()
        result = check.run(dataset)
        self.assertTrue(result.passed, f"Non expected check result for check '{type(check)}' with message '{result.message}'")


if __name__ == '__main__':
    main()
